@@ -1,8 +1,10 @@
 """Checker Manager and Checker classes."""
 import collections
 import errno
+import itertools
 import logging
 import signal
+import sys
 import tokenize
 from typing import Dict, List, Optional, Tuple
 
@@ -34,6 +36,18 @@ SERIAL_RETRY_ERRNOS = {
 }
 
 
+def _multiprocessing_is_fork():  # type () -> bool
+    """Class state is only preserved when using the `fork` strategy."""
+    if sys.version_info >= (3, 4):
+        return (
+            multiprocessing
+            # https://github.com/python/typeshed/pull/3415
+            and multiprocessing.get_start_method() == "fork"  # type: ignore
+        )
+    else:
+        return multiprocessing and not utils.is_windows()
+
+
 class Manager(object):
     """Manage the parallelism and checker instances for each plugin and file.
 
@@ -72,6 +86,7 @@ class Manager(object):
         self.options = style_guide.options
         self.checks = checker_plugins
         self.jobs = self._job_count()
+        self._all_checkers = []  # type: List[FileChecker]
         self.checkers = []  # type: List[FileChecker]
         self.statistics = {
             "files": 0,
@@ -79,6 +94,9 @@ class Manager(object):
             "physical lines": 0,
             "tokens": 0,
         }
+        self.exclude = tuple(
+            itertools.chain(self.options.exclude, self.options.extend_exclude)
+        )
 
     def _process_statistics(self):
         for checker in self.checkers:
@@ -97,26 +115,13 @@ class Manager(object):
         # - we're processing a diff, which again does not work well with
         #   multiprocessing and which really shouldn't require multiprocessing
         # - the user provided some awful input
-        if not multiprocessing:
+        if not _multiprocessing_is_fork():
             LOG.warning(
                 "The multiprocessing module is not available. "
                 "Ignoring --jobs arguments."
             )
             return 0
 
-        if (
-            utils.is_windows()
-            and not utils.can_run_multiprocessing_on_windows()
-        ):
-            LOG.warning(
-                "The --jobs option is not available on Windows due to"
-                " a bug (https://bugs.python.org/issue27649) in "
-                "Python 2.7.11+ and 3.3+. We have detected that you "
-                "are running an unsupported version of Python on "
-                "Windows. Ignoring --jobs arguments."
-            )
-            return 0
-
         if utils.is_using_stdin(self.arguments):
             LOG.warning(
                 "The --jobs option is not compatible with supplying "
@@ -132,19 +137,12 @@ class Manager(object):
             return 0
 
         jobs = self.options.jobs
-        if jobs != "auto" and not jobs.isdigit():
-            LOG.warning(
-                '"%s" is not a valid parameter to --jobs. Must be one '
-                'of "auto" or a numerical value, e.g., 4.',
-                jobs,
-            )
-            return 0
 
         # If the value is "auto", we want to let the multiprocessing library
         # decide the number based on the number of CPUs. However, if that
         # function is not implemented for this particular value of Python we
         # default to 1
-        if jobs == "auto":
+        if jobs.is_auto:
             try:
                 return multiprocessing.cpu_count()
             except NotImplementedError:
@@ -152,7 +150,7 @@ class Manager(object):
 
         # Otherwise, we know jobs should be an integer and we can just convert
         # it to an integer
-        return int(jobs)
+        return jobs.n_jobs
 
     def _handle_results(self, filename, results):
         style_guide = self.style_guide
@@ -187,7 +185,7 @@ class Manager(object):
 
         return utils.matches_filename(
             path,
-            patterns=self.options.exclude,
+            patterns=self.exclude,
             log_message='"%(path)s" has %(whether)sbeen excluded',
             logger=LOG,
         )
@@ -230,17 +228,15 @@ class Manager(object):
             ) or is_stdin
 
         checks = self.checks.to_dictionary()
-        checkers = (
+        self._all_checkers = [
             FileChecker(filename, checks, self.options)
             for argument in paths
             for filename in utils.filenames_from(
                 argument, self.is_path_excluded
             )
             if should_create_file_checker(filename, argument)
-        )
-        self.checkers = [
-            checker for checker in checkers if checker.should_process
         ]
+        self.checkers = [c for c in self._all_checkers if c.should_process]
         LOG.info("Checking %d files", len(self.checkers))
 
     def report(self):
@@ -256,7 +252,7 @@ class Manager(object):
             tuple(int, int)
         """
         results_reported = results_found = 0
-        for checker in self.checkers:
+        for checker in self._all_checkers:
             results = sorted(
                 checker.results, key=lambda tup: (tup[1], tup[2])
             )
@@ -304,9 +300,7 @@ class Manager(object):
 
         for checker in self.checkers:
             filename = checker.display_name
-            checker.results = sorted(
-                final_results[filename], key=lambda tup: (tup[2], tup[2])
-            )
+            checker.results = final_results[filename]
             checker.statistics = final_statistics[filename]
 
     def run_serial(self):
@@ -363,7 +357,7 @@ class FileChecker(object):
         :param options:
             Parsed option values from config and command-line.
         :type options:
-            optparse.Values
+            argparse.Namespace
         """
         self.options = options
         self.filename = filename
@@ -384,7 +378,7 @@ class FileChecker(object):
             self.should_process = not self.processor.should_ignore_file()
             self.statistics["physical lines"] = len(self.processor.lines)
 
-    def __repr__(self):
+    def __repr__(self):  # type: () -> str
         """Provide helpful debugging representation."""
         return "FileChecker for {}".format(self.filename)
 
@@ -403,20 +397,20 @@ class FileChecker(object):
             self.report("E902", 0, 0, message)
             return None
 
-    def report(self, error_code, line_number, column, text, line=None):
-        # type: (str, int, int, str, Optional[str]) -> str
+    def report(self, error_code, line_number, column, text):
+        # type: (Optional[str], int, int, str) -> str
         """Report an error by storing it in the results list."""
         if error_code is None:
             error_code, text = text.split(" ", 1)
 
-        physical_line = line
         # If we're recovering from a problem in _make_processor, we will not
         # have this attribute.
-        if not physical_line and getattr(self, "processor", None):
-            physical_line = self.processor.line_for(line_number)
+        if hasattr(self, "processor"):
+            line = self.processor.noqa_line_for(line_number)
+        else:
+            line = None
 
-        error = (error_code, line_number, column, text, physical_line)
-        self.results.append(error)
+        self.results.append((error_code, line_number, column, text, line))
         return error_code
 
     def run_check(self, plugin, **arguments):
@@ -480,7 +474,7 @@ class FileChecker(object):
             column -= column_offset
         return row, column
 
-    def run_ast_checks(self):
+    def run_ast_checks(self):  # type: () -> None
         """Run all checks expecting an abstract syntax tree."""
         try:
             ast = self.processor.build_ast()
@@ -499,7 +493,7 @@ class FileChecker(object):
                 runner = checker.run()
             except AttributeError:
                 runner = checker
-            for (line_number, offset, text, check) in runner:
+            for (line_number, offset, text, _) in runner:
                 self.report(
                     error_code=None,
                     line_number=line_number,
@@ -520,8 +514,9 @@ class FileChecker(object):
             self.processor.update_checker_state_for(plugin)
             results = self.run_check(plugin, logical_line=logical_line) or ()
             for offset, text in results:
-                offset = find_offset(offset, mapping)
-                line_number, column_offset = offset
+                line_number, column_offset = find_offset(offset, mapping)
+                if line_number == column_offset == 0:
+                    LOG.warning("position of error out of bounds: %s", plugin)
                 self.report(
                     error_code=None,
                     line_number=line_number,
@@ -531,7 +526,7 @@ class FileChecker(object):
 
         self.processor.next_logical_line()
 
-    def run_physical_checks(self, physical_line, override_error_line=None):
+    def run_physical_checks(self, physical_line):
         """Run all checks for a given physical line.
 
         A single physical check may return multiple errors.
@@ -554,15 +549,11 @@ class FileChecker(object):
 
                 for result_single in result:
                     column_offset, text = result_single
-                    error_code = self.report(
+                    self.report(
                         error_code=None,
                         line_number=self.processor.line_number,
                         column=column_offset,
                         text=text,
-                        line=(override_error_line or physical_line),
-                    )
-                    self.processor.check_physical_error(
-                        error_code, physical_line
                     )
 
     def process_tokens(self):
@@ -640,9 +631,7 @@ class FileChecker(object):
             line_no = token[2][0]
             with self.processor.inside_multiline(line_number=line_no):
                 for line in self.processor.split_line(token):
-                    self.run_physical_checks(
-                        line + "\n", override_error_line=token[4]
-                    )
+                    self.run_physical_checks(line + "\n")
 
 
 def _pool_init():
@@ -667,11 +656,17 @@ def _run_checks(checker):
 
 
 def find_offset(offset, mapping):
+    # type: (int, processor._LogicalMapping) -> Tuple[int, int]
     """Find the offset tuple for a single offset."""
     if isinstance(offset, tuple):
         return offset
 
-    for token_offset, position in mapping:
+    for token in mapping:
+        token_offset = token[0]
         if offset <= token_offset:
+            position = token[1]
             break
+    else:
+        position = (0, 0)
+        offset = token_offset = 0
     return (position[0], position[1] + offset - token_offset)
